"""
Copyright 2021-2023 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from logging import Logger
from typing import List, Union
from influxdb_client import InfluxDBClient, Point
from influxdb_client.domain.write_precision import WritePrecision
from .entities.device_measurement import DeviceMeasurement
from .entities.gateway_measurement import GatewayMeasurement
from .entities.sensor_measurement import SensorMeasurement
from .influx_client_interface import InfluxClientInterface
from .custom_logging import Logging

__all__ = [
	'InfluxClient'
]


class InfluxClient(InfluxClientInterface):
	"""Influxdb (1.8/2.0+) client wrapper class."""

	def __init__(self, config: dict) -> None:
		"""
		Construct attributes for Influxdb client object.

		Args:
			config (dict): influx db client configuration parameters
		"""
		self.logger: Logger = Logging.get_logger()
		self.url: str = config['url']
		self.token: str = config['token']
		self.org: str = config['org']
		self.client = InfluxDBClient(
			url=self.url,
			token=self.token,
			org=self.org
		)
		self.writer = self.client.write_api()

	def write(self, bucket: str, measurements: List[Union[DeviceMeasurement, GatewayMeasurement, SensorMeasurement]]):
		"""
		Write measurements to database and logs information.

		Args:
			bucket (str): bucket
			measurements (List[Union[DeviceMeasurement, GatewayMeasurement, SensorMeasurement]]): list of measurement
				objects to be written to database
		"""
		for measurement in measurements:
			record: Point = Point.from_dict(measurement.serialize())
			self.writer.write(bucket=bucket, write_precision=WritePrecision.NS, record=record)
			self.logger.debug(Logging.log_measurement(measurement))
