"""
Copyright 2021-2023 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from logging import Logger
from typing import List, Union
from urllib.parse import urlparse
from influxdb import InfluxDBClient
from .entities.device_measurement import DeviceMeasurement
from .entities.gateway_measurement import GatewayMeasurement
from .entities.sensor_measurement import SensorMeasurement
from .exceptions.invalid_credentials_exception import InvalidCredentialsException
from .influx_client_interface import InfluxClientInterface
from .custom_logging import Logging

__all__ = [
	'InfluxClientLegacy'
]


class InfluxClientLegacy(InfluxClientInterface):
	"""Legacy Influxdb (1.x) client wrapper class."""

	# pylint: disable=too-many-instance-attributes

	def __init__(self, config: dict) -> None:
		"""
		Construct attributes for legacy Influxdb client object.

		Args:
			config (dict): influx db client configuration parameters
		"""
		self.logger: Logger = Logging.get_logger()
		self.url: str = config['url']
		self.user: str = config['user']
		self.password: str = config['password']
		self.org: str = config['org']
		if len(self.user) == 0 or len(self.password) == 0:
			raise InvalidCredentialsException('Invalid credentials, username and password must not be empty when using '
												'legacy client.')
		url_tokens = urlparse(self.url)
		self.client: InfluxDBClient = InfluxDBClient(
			host=url_tokens.hostname or 'localhost',
			port=url_tokens.port or 8086,
			path=url_tokens.path,
			username=self.user,
			password=self.password,
		)
		if not any(user.get('user') == self.user for user in self.client.get_list_users()):
			self.client.create_user(self.user, self.password)
		dbs = self.client.get_list_database()
		existing_buckets = [db.get('name') for db in dbs]
		buckets = [config['buckets']['gateway'], config['buckets']['devices'], config['buckets']['sensors']]
		for bucket in buckets:
			if bucket in existing_buckets:
				continue
			self.client.create_database(bucket)
			self.client.grant_privilege('all', bucket, self.user)

	def write(self, bucket: str, measurements: List[Union[DeviceMeasurement, GatewayMeasurement, SensorMeasurement]]):
		"""
		Write measurements to database and logs information.

		Args:
			bucket (str): bucket
			measurements (List[Union[DeviceMeasurement, GatewayMeasurement, SensorMeasurement]]): list of measurement
				objects to be written to database
		"""
		self.client.switch_database(bucket)
		for measurement in measurements:
			point = measurement.serialize()
			self.client.write_points([point], time_precision='n')
			self.logger.debug(Logging.log_measurement(measurement))
