"""
Copyright 2021-2023 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from typing import List

from .enums.charger_data import ChargerData
from .entities.device_measurement import DeviceMeasurement
from .entities.gateway_measurement import GatewayMeasurement
from .entities.sensor_measurement import SensorMeasurement


class MessageParser:
	"""Collection of functions for parsing of incoming messages."""

	@staticmethod
	def parse_charger(data: dict, gw_id: str) -> List[GatewayMeasurement]:
		"""
		Parse Daemon Charger data response message and return list of gateway measurement objects.

		Args:
			data (dict): GW MCU charger response data
			gw_id (str): Gateway ID

		Returns:
			List[GatewayMeasurement]: List of gateway measurement objects
		"""
		measurements = []
		for member in ChargerData:
			measurements.append(
				GatewayMeasurement(
					gw_id=gw_id,
					name=f'charger_{member.value}',
					value=data[member.value]
				)
			)
		return measurements

	@staticmethod
	def parse_sensor_data_device(data: dict, gw_id: str) -> List[DeviceMeasurement]:
		"""
		Parse Daemon API IQMESH Network Sensor Data response message and return list of device measurement objects.

		Args:
			data (dict): IQMESH Network Sensor Data response data
			gw_id (str): Gateway ID

		Returns:
			List[DeviceMeasurement]: List of device measurement objects
		"""
		measurements = []
		for device in data['devices']:
			measurements.append(
				DeviceMeasurement(
					gw_id=gw_id,
					addr=device['address'],
					mid=device['mid'],
					name='rssi',
					value=device['rssi']
				)
			)
		return measurements

	@staticmethod
	def parse_sensor_data_sensor(data: dict, gw_id: str) -> List[SensorMeasurement]:
		"""
		Parse Daemon API IQMESH Network Sensor Data response message and return list of sensor measurement objects.

		Args:
			data (dict): IQMESH Network Sensor Data response data
			gw_id (str): Gateway ID

		Returns:
			List[DeviceMeasurement]: List of sensor measurement objects
		"""
		measurements = []
		for device in data['devices']:
			addr = device['address']
			mid = device['mid']
			for sensor in device['sensors']:
				measurements.append(
					SensorMeasurement(
						gw_id=gw_id,
						addr=addr,
						mid=mid,
						idx=sensor['index'],
						type=sensor['type'],
						name=sensor['name'],
						unit=sensor['unit'],
						value=sensor['value']
					)
				)
		return measurements
